/*
 * Bestimmung der Netzfrequenz
 * Copyright 2015-2022 by Ralf Th. Pietsch <ratopi@abwesend.de>
 * LICENSE: MIT
 *
 * https://www.netzfrequenzmessung.de/index.htm 
 * Da gibt's tolle Infos zum Thema.
 *
 * W. Mikuteit <DK3WM@WEB.de>
 *  23. Okt. 2022   
 *    - I2C OLED SH1106 - Anzeige ergänzt
 *    - Beschreibung angefügt
 * 
 * Kurzbeschreibung:
 * DEMO - mal kein BLINK oder HALLO WELT :-)
 * 
 * Wenn DEBUG am Codeanfang definiert ist, wird das Ergebniss auch
 * seriell ausgegeben (Monitoring).
 * 
 * Am analog Eingang (A0 beim NANO) stecken ca. 10 - 15 cm Draht.
 * Die darüber eingekoppelte "Brummspannung" des Stromnetzes ist
 * für die Messung ausreichend.
 *
 * Mit der Konstanten "PROBE_COUNT" wird die Anzahl der für die
 * Messung auszuwertdenden Perioden festgelegt. 
 * Beim Wert 100 werden demzufolge 2 Sekunden ausgewertet. 
 * Diese Einstellung war bei mir stabil.
 * 
 * Die Zeit für die Messung der "PROBE_COUNT"-Perioden wird in 
 * Microsekunden ermittelt.
 * Daraus lässt sich dann die Frequenz errechnen und ausgeben.
 * 
 * Mit der Konstanten "KORREKTUR" lassen sich Abweichungnen ausgleichen.
 * (Taktfrequenz, Temperatureinfluss, Laufzeit u.s.w)
 * 
 * VIEL SPASS!
*/

#include <Arduino.h>
#include <U8x8lib.h>

#ifdef U8X8_HAVE_HW_SPI
#include <SPI.h>
#endif
#ifdef U8X8_HAVE_HW_I2C
#include <Wire.h>
#endif

// Aktivieren für seriellen Output/debugging
#define DEBUG 
#define PROBE_COUNT 100
#define MICROS 1000000.
//#define KORREKTUR 1.0   
#define KORREKTUR 1.00889
int min;
int max;
int val;
int i, j;
float freq;
long start;
long end;

// Das i2C-Display kommt an +5V und GND, und an SCL und SDA vom NANO
// Hardware i2C
U8X8_SH1106_128X64_NONAME_HW_I2C u8x8(/* reset=*/ U8X8_PIN_NONE);

void setup(void)
{
  analogReference( INTERNAL );
  
  #ifdef DEBUG
  Serial.begin( 9600 );
  Serial.println( "reading 10000 samples ..." );
  #endif
  
  for ( i = 0; i < 10000; i++ )
  {
      analogRead( A0 );
  }
  
  #ifdef DEBUG
  Serial.println( "waiting for first period");
  #endif
  
  period();
  end = micros();
  
  #ifdef DEBUG
  Serial.println( "starting..." );
  #endif

  u8x8.begin();
  // Display - alle statischen Infos anzeigen
  u8x8.setFont(u8x8_font_amstrad_cpc_extended_f);    
  u8x8.clear();
  u8x8.inverse();
  u8x8.print(" Netzfrequenz ");
  u8x8.setFont(u8x8_font_chroma48medium8_r);  
  u8x8.noInverse();
  u8x8.setCursor(0,1);
  u8x8.drawString(0, 1, "");
  u8x8.setCursor(0, 6);
  u8x8.drawString(0, 6, "      Hz");  
}

void loop(void)
{
  start = end;
  min = 1024;
  max = 0;
  for ( i = 0; i < PROBE_COUNT; i++ )
  {
    period();
  }
  end = micros();
  freq = ( ( PROBE_COUNT * MICROS * KORREKTUR ) / ( end - start ) );

#ifdef DEBUG
  Serial.print( min );
  Serial.print( " " );
  Serial.print( max );
  Serial.print( " " );
  Serial.println( freq, 3 );
#endif
   
  u8x8.setFont(u8x8_font_px437wyse700b_2x2_r);
  u8x8.setCursor(1, 3);
  u8x8.print(freq, 3);
}

void period()
{
  while ( analogReadPlus() > 50 );
  while ( analogReadPlus() < 100 );
}

int analogReadPlus() 
{
  val = analogRead( A0 );
  if ( val < min ) min = val;
  if ( val > max ) max = val;
  return val;
}
